@@ -54,7 +54,7 @@ KEYS = _NamedInts(
 _BATTERY_ATTENTION_LEVEL = 5
 
 # If no updates have been receiver from the device for a while, ping the device
-# and update it status accordinly.
+# and update it status accordingly.
 # _STATUS_TIMEOUT = 5 * 60  # seconds
 _LONG_SLEEP = 15 * 60  # seconds
 
@@ -176,8 +176,16 @@ class DeviceStatus(dict):
 
 		if level is None:
 			# Some notifications may come with no battery level info, just
-			# charging state info, so assume the level is unchanged.
-			level = self.get(KEYS.BATTERY_LEVEL)
+			# charging state info, so do our best to infer a level (even if it is just the last level)
+			# It is not always possible to do this well
+			if status == _hidpp20.BATTERY_STATUS.full:
+				level = _hidpp10.BATTERY_APPOX.full
+			elif status in (_hidpp20.BATTERY_STATUS.almost_full, _hidpp20.BATTERY_STATUS.recharging):
+				level = _hidpp10.BATTERY_APPOX.good
+			elif status == _hidpp20.BATTERY_STATUS.slow_recharge:
+				level = _hidpp10.BATTERY_APPOX.low
+			else:
+				level = self.get(KEYS.BATTERY_LEVEL)
 		else:
 			assert isinstance(level, int)
 
@@ -185,13 +193,14 @@ class DeviceStatus(dict):
 		old_level, self[KEYS.BATTERY_LEVEL] = self.get(KEYS.BATTERY_LEVEL), level
 		old_status, self[KEYS.BATTERY_STATUS] = self.get(KEYS.BATTERY_STATUS), status
 
-		charging = status in (_hidpp20.BATTERY_STATUS.recharging, _hidpp20.BATTERY_STATUS.slow_recharge)
+		charging = status in (_hidpp20.BATTERY_STATUS.recharging, _hidpp20.BATTERY_STATUS.almost_full,
+				      _hidpp20.BATTERY_STATUS.full, _hidpp20.BATTERY_STATUS.slow_recharge)
 		old_charging, self[KEYS.BATTERY_CHARGING] = self.get(KEYS.BATTERY_CHARGING), charging
 
 		changed = old_level != level or old_status != status or old_charging != charging
 		alert, reason = ALERT.NONE, None
 
-		if _hidpp20.BATTERY_OK(status) and level > _BATTERY_ATTENTION_LEVEL:
+		if _hidpp20.BATTERY_OK(status) and ( level is None or level > _BATTERY_ATTENTION_LEVEL ):
 			self[KEYS.ERROR] = None
 		else:
 			_log.warn("%s: battery %d%%, ALERT %s", self._device, level, status)
@@ -202,7 +211,7 @@ class DeviceStatus(dict):
 			if isinstance(level, _NamedInt):
 				reason = _("Battery: %(level)s (%(status)s)") % { 'level': _(level), 'status': _(status) }
 			else:
-				reason = _("Battery: %(percent)d%% (%(status)s)") % { 'percent': level, 'status': _(status) }
+				reason = _("Battery: %(percent)d%% (%(status)s)") % { 'percent': level, 'status': status.name }
 
 		if changed or reason:
 			# update the leds on the device, if any
@@ -223,7 +232,7 @@ class DeviceStatus(dict):
 			# broadcasting it's battery status anyway, it will just take a little while.
 			# However, when the device has just been detected, it will not show
 			# any battery status for a while (broadcasts happen every 90 seconds).
-			if battery is None and _hidpp20.FEATURE.SOLAR_DASHBOARD in d.features:
+			if battery is None and d.features and _hidpp20.FEATURE.SOLAR_DASHBOARD in d.features:
 				d.feature_request(_hidpp20.FEATURE.SOLAR_DASHBOARD, 0x00, 1, 1)
 				return
 
@@ -261,7 +270,8 @@ class DeviceStatus(dict):
 
 					# Devices lose configuration when they are turned off,
 					# make sure they're up-to-date.
-					# _log.debug("%s settings %s", d, d.settings)
+					if _log.isEnabledFor(_DEBUG):
+						_log.debug("%s pushing device settings %s", d, d.settings)
 					for s in d.settings:
 						s.apply()
 
